### Program head ###
import os
import numpy as np
import math as m
import matplotlib
import matplotlib.pyplot as plt 
import seaborn as sns
from scipy import interpolate
matplotlib.rcParams['svg.fonttype'] = 'none'

sns.set_style('ticks')
sns.set_context('paper')



###  Input parameters for simulation  ###
# mintorsion,maxtorsion,ktorsionstep = np.array([180, 180, 10])/180*np.pi # Torsional angle of C-H bond around C-C bond, min, max, incmn
torsion = 180 # Torsional angle of C-H bond around C-C bond
nalpha = 8 # nalpha,nbeta and ngamma should be multiples of 4
nbeta = 4
ngamma = 8
ntd = 20 # Number of time domain points per rotor cycle
omigar = 10.5 # MAS frq in kHz
idipcsyn = 0 # Dipolar and CS both(2); no dip(1), no CS(0).
idqyn = 0 # Double Quantum factor yes(1)/no(0)
cs11,cs22,cs33 = np.array([-5.9, 0, 5.9]) # Orientation of cc bond in principal system of C_alpha
deltadp1 = 26.54*np.arange(0.2,1.2,0.2) #!!!! List of X-H dipolar coupling, 1-bond, all correction included
deltadp2=1.2
anhalpha,anhbeta,anhgamma = np.array([0.0, 60.0, 0.0])/180*np.pi # alpha,beta,gamma of N-C in NH CSA system
chalpha,chbeta = np.array([0.0, 109.5])/180*np.pi # Orientation of C-C bond in C-H dip system? alpha and beta of N-C in C-H  system

cbhalpha, cbhbeta, cbhgamma = np.array([0.0, 25.0, 0.0])/180*np.pi # Unknown angles
omigat2 = 0.0 # Unknown parameter






### Program body start here
def rotation(alpha,beta,gamma,sbefore):
    # Similarity transformation of matrix sbefore to safter using the rotation matrix of mehring p.292
    # The inverse of a rotation matrix is its transposed. All angles in degrees!
    
    ca,cb,cg = np.cos([alpha,beta,gamma])
    sa,sb,sg = np.sin([alpha,beta,gamma])
    R = np.array(((ca*cb*cg-sa*sg,sa*cb*cg+ca*sg,-sb*cg),
                  (-ca*cb*sg-sa*cg,-sa*cb*sg+ca*cg,sb*sg),
                  (ca*sb,sa*sb,cb)))
    safter = np.matmul(np.matmul(R,sbefore),R.T)
    return safter

def phi(sigma4,alpha,beta,gamma):
    sigdiag1 = sigma4[:,0,0]+sigma4[:,1,1]-2*sigma4[:,2,2]
    sigdiag2 = sigma4[:,1,1]-sigma4[:,0,0]
    sa,sa2,sb,sb2 = np.sin([alpha,2*alpha,beta,2*beta])
    ca,ca2,cb,cb2 = np.cos([alpha,2*alpha,beta,2*beta])
    cbsq = cb*cb
    # c1,c2 and s1,s2 are c1^nd,c2^nd etc in table 3.1 of reference, 2*np.sqrt(2)/3 = 0.942809
    c1 = 0.942809*(0.25*sigdiag1*sb2-0.25*sigdiag2*sb2*ca2
                         +sigma4[:,0,1]/2*sb2*sa2
                         +sigma4[:,0,2]*cb2*ca
                         +sigma4[:,1,2]*cb2*sa)

    c2 = 1/3*(-.5*sigdiag1*sb*sb-.5*sigdiag2*(1+cbsq)*ca2
              +sigma4[:,0,1]*(1+cbsq)*sa2
              -sigma4[:,0,2]*sb2*ca
              -sigma4[:,1,2]*sb2*sa)

    s1 = 0.942809*(.5*sigdiag2*sb*sa2
                         +sigma4[:,0,1]*sb*ca2
                         -sigma4[:,0,2]*cb*sa
                         +sigma4[:,1,2]*cb*ca)

    s2 = 2/3*(.5*sigdiag2*cb*sa2
              +sigma4[:,0,1]*cb*ca2
              +sigma4[:,0,2]*sb*sa
              -sigma4[:,1,2]*sb*ca)

    phi = (c1*m.sin(gamma)+c2/2*m.sin(2*gamma)
           -s1*m.cos(gamma)-s2/2*m.cos(2*gamma))/omigar
    
    return phi

def dqfactor(alpha,beta,gamma):
    # double quantum factor
    delta = 2 # matrix for C-C homonuclear coupling?????
    c1 = -delta*0.707107*m.sin(beta) # 0.707107: sqrt(2)/2
    s1tilde = -c1*m.sin(gamma)
    dqtemp = m.sin(2*s1tilde/omigar)
    dqfactor = dqtemp*dqtemp
    return dqfactor


#    Initialize
gretemp2,gimtemp2,gretemp4,gimtemp4,g2re,g2im,g1re,g1im = (np.zeros(ntd) for i in range(8)) # initialize g arrays length ntd
gretemp1,gimtemp1 = (np.zeros((ntd,ngamma)) for i in range(2)) # initialize re&im of gtemp1 for gamma
gretemp3,gimtemp3 = (np.zeros((ntd,nbeta)) for i in range(2)) # initialize re&im of gtemp3 for beta
gretemp5,gimtemp5 = (np.zeros((ntd,nalpha)) for i in range(2)) # initialize re&im of gtemp5 for alpha
# gre,gim = (np.zeros((int((maxtorsion-mintorsion)/ktorsionstep)+1,ntd)) for i in range(2))
gre,gim = (np.zeros((len(deltadp1),ntd)) for i in range(2))
legend = []

#  calculate sum and dif tensor value.
for i in range(len(deltadp1)):
    deltadp = deltadp1[i]
#     for i in np.arange(int((maxtorsion-mintorsion)/ktorsionstep)+1):
#     torsion = mintorsion+i*ktorsionstep
    # getsigma in original fortran code, 
    # for calculating double quantum sum and difference tensors of N-H, C-H, and C-H dipolar couplings.
    cbnyl = rotation(anhalpha,anhbeta,anhgamma,np.diag([cs11,cs22,cs33]))
    dip = rotation(chalpha,chbeta,torsion,np.diag([-0.5*deltadp,-0.5*deltadp,deltadp]))
    dip2 = rotation(cbhalpha,cbhbeta,cbhgamma,np.diag([-0.5*deltadp2,-0.5*deltadp2,deltadp2]))

    if idipcsyn ==2:
        sm  =cbnyl+dip+dip2
        dif =cbnyl-dip-dip2
        smdf=cbnyl+dip-dip2
        dfsm=cbnyl-dip+dip2
    elif idipcsyn ==1:
        sm  =cbnyl
        dif =cbnyl
        smdf=cbnyl
        dfsm=cbnyl
    elif idipcsyn ==0:
        sm  =dip+dip2
        dif =-dip-dip2
        smdf=dip-dip2
        dfsm=-dip+dip2

    if idqyn == 0:
        dq=1
    else:
        dq=dqfactor(alpha,beta,gamma)

    coupl = np.array([sm,dif,smdf,dfsm])
    # getg2 in original fortran code 
    # Calculate time signal, perform 3-D parameter integration by doing 1-D parameter integration repeatedly.
    for ialpha in range(1,nalpha+1):
        alpha = 2*ialpha/nalpha*np.pi
        for ibeta in range(1,nbeta+1):
            beta = ibeta/nbeta*np.pi
            for igamma in range(1,ngamma+1):
                gamma = 2*igamma/ngamma*np.pi           
                # getg1 in fortran, for getting g(t), the time signal before integration with alpha,gamma,beta.
                phi0 = phi(coupl,alpha,beta,gamma)
                for itime in range(ntd):
                    phtime=itime/ntd*2*np.pi
                    gammapm=gamma+phtime
                    phipm=phi(coupl,alpha,beta,gammapm)
                    g1re[itime]=dq*sum(np.cos(phipm-phi0))
                    g1im[itime]=dq*sum(np.sin(phipm-phi0))
                # End of getg1
                gretemp1[:,igamma-1]=g1re*2.*np.pi/ngamma
                gimtemp1[:,igamma-1]=g1im*2.*np.pi/ngamma
            #end igamma loop
            gretemp2 = np.sum(gretemp1,axis=1)
            gimtemp2 = np.sum(gimtemp1,axis=1)
            gretemp3[:,ibeta-1] = gretemp2*np.sin(beta)*np.pi/nbeta
            gimtemp3[:,ibeta-1] = gimtemp2*np.sin(beta)*np.pi/nbeta
        #end ibeta loop
        gretemp4 = np.sum(gretemp3,axis=1)
        gimtemp4 = np.sum(gimtemp3,axis=1)
        gretemp5[:,ialpha-1]=gretemp4*2*np.pi/nalpha
        gimtemp5[:,ialpha-1]=gimtemp4*2*np.pi/nalpha
    #end ialpha loop
    g2re = np.sum(gretemp5,axis=1)
    g2im = np.sum(gimtemp5,axis=1)
    coeft2=omigat2/(ntd*omigar)
    gre[i,:] = g2re*np.exp(-np.arange(1,ntd+1)*coeft2)
    gim[i,:] = g2im*np.exp(-np.arange(1,ntd+1)*coeft2)
    legend.append(str(round(deltadp,2)))
    

gre_norm = 1/gre[:,0]
gre *= gre_norm[:,None]
# gim_norm = 1/gim[:,0]
# gim *= gre_norm[:,None]
t_plot = np.arange(ntd)/ntd

for i in range(gre.shape[0]):
    print('Dip coupling = '+legend[i])
    print('Dipshift dephase:\n')
    for j in range(ntd):
        print(gre[i,j])
    print('\n')

fig, ax = plt.subplots()
for i in range(gre.shape[0]):
    tck = interpolate.splrep(t_plot, gre[i,:], s=0)
    t_new = np.arange(ntd*20)/(ntd*20)
    gre_smooth = interpolate.splev(t_new, tck, der=0)
    ax.plot(t_new, gre_smooth, label=legend[i])
    
legend = ax.legend(loc='best', fontsize='medium')
plt.xlim([0, 1])
plt.ylim([0, 1])
plt.xlabel('Dephasing time t/tr')
plt.title('DipX2 Sim, assuming rigid limit 23kHz')
plt.savefig('Your file name.svg') # Save the figure